@@ -25,7 +25,10 @@
 
 G_BEGIN_DECLS
 
-#include "gsystem-local-alloc.h"
+#include <gsystem-local-alloc.h>
+#include <gsystem-file-utils.h>
+#include <gsystem-shutil.h>
+#include <gsystem-subprocess.h>
 
 G_END_DECLS
 
