@@ -69,7 +69,8 @@ ot_util_variant_save (GFile *dest,
   ot_lobj GOutputStream *out = NULL;
   gsize bytes_written;
   
-  out = (GOutputStream*)g_file_replace (dest, NULL, 0, FALSE, cancellable, error);
+  out = (GOutputStream*)g_file_replace (dest, NULL, FALSE, G_FILE_CREATE_REPLACE_DESTINATION,
+                                        cancellable, error);
   if (!out)
     goto out;
 
@@ -91,13 +92,7 @@ ot_util_variant_save (GFile *dest,
 GVariant *
 ot_util_variant_take_ref (GVariant *variant)
 {
-#if GLIB_CHECK_VERSION(2,32,0) && !defined(OSTREE_GLIB_TARGET_MIN)
   return g_variant_take_ref (variant);
-#else
-  if (g_variant_is_floating (variant))
-    return g_variant_ref_sink (variant);
-  return variant;
-#endif
 }
 
 /**
@@ -107,25 +102,31 @@ ot_util_variant_take_ref (GVariant *variant)
  * Note the returned @out_variant is not floating.
  */
 gboolean
-ot_util_variant_map (GFile *src,
+ot_util_variant_map (GFile              *src,
                      const GVariantType *type,
-                     GVariant **out_variant,
-                     GError  **error)
+                     gboolean            trusted,
+                     GVariant          **out_variant,
+                     GError            **error)
 {
   gboolean ret = FALSE;
   const char *path = NULL;
   ot_lvariant GVariant *ret_variant = NULL;
   GMappedFile *mfile = NULL;
+  int fd;
 
-  path = ot_gfile_get_path_cached (src);
-  mfile = g_mapped_file_new (path, FALSE, error);
+  path = gs_file_get_path_cached (src);
+  if (!ot_unix_open_noatime (path, &fd, error))
+    goto out;
+  mfile = g_mapped_file_new_from_fd (fd, FALSE, error);
   if (!mfile)
     goto out;
+  if (!ot_unix_close (fd, error))
+    goto out;
 
   ret_variant = g_variant_new_from_data (type,
                                          g_mapped_file_get_contents (mfile),
                                          g_mapped_file_get_length (mfile),
-                                         FALSE,
+                                         trusted,
                                          (GDestroyNotify) g_mapped_file_unref,
                                          mfile);
   mfile = NULL;
