@@ -31,13 +31,17 @@
 typedef struct {
   OstreeRepo  *repo;
   GFile *ostree_dir;
+  char  *osname;
+  GFile *osname_dir;
 } OtAdminDeploy;
 
 static gboolean opt_no_kernel;
 static gboolean opt_force;
+static gboolean opt_host_kernel;
 
 static GOptionEntry options[] = {
   { "no-kernel", 0, 0, G_OPTION_ARG_NONE, &opt_no_kernel, "Don't update kernel related config (initramfs, bootloader)", NULL },
+  { "host-kernel", 0, 0, G_OPTION_ARG_NONE, &opt_host_kernel, "Use currently booted kernel, not kernel from tree", NULL },
   { "force", 0, 0, G_OPTION_ARG_NONE, &opt_force, "Overwrite any existing deployment", NULL },
   { NULL }
 };
@@ -61,21 +65,18 @@ update_current (OtAdminDeploy      *self,
 {
   gboolean ret = FALSE;
   ot_lobj GFile *current_path = NULL;
-  ot_lobj GFile *current_etc_path = NULL;
   ot_lobj GFile *previous_path = NULL;
   ot_lobj GFile *tmp_current_path = NULL;
-  ot_lobj GFile *tmp_current_etc_path = NULL;
   ot_lobj GFile *tmp_previous_path = NULL;
   ot_lobj GFileInfo *previous_info = NULL;
   ot_lfree char *relative_current = NULL;
   ot_lfree char *relative_current_etc = NULL;
   ot_lfree char *relative_previous = NULL;
 
-  current_path = g_file_get_child (self->ostree_dir, "current");
-  current_etc_path = g_file_get_child (self->ostree_dir, "current-etc");
-  previous_path = g_file_get_child (self->ostree_dir, "previous");
+  current_path = g_file_get_child (self->osname_dir, "current");
+  previous_path = g_file_get_child (self->osname_dir, "previous");
 
-  relative_current = g_file_get_relative_path (self->ostree_dir, deploy_target);
+  relative_current = g_file_get_relative_path (self->osname_dir, deploy_target);
   g_assert (relative_current);
   relative_current_etc = g_strconcat (relative_current, "-etc", NULL);
 
@@ -85,55 +86,44 @@ update_current (OtAdminDeploy      *self,
 
       if (g_file_equal (current_deployment, deploy_target))
         {
-          g_print ("ostadmin: %s already points to %s\n", ot_gfile_get_path_cached (current_path),
+          g_print ("ostadmin: %s already points to %s\n", gs_file_get_path_cached (current_path),
                    relative_current);
           return TRUE;
         }
 
-      tmp_previous_path = g_file_get_child (self->ostree_dir, "tmp-previous");
-      (void) ot_gfile_unlink (tmp_previous_path, NULL, NULL);
+      tmp_previous_path = g_file_get_child (self->osname_dir, "tmp-previous");
+      (void) gs_file_unlink (tmp_previous_path, NULL, NULL);
 
-      relative_previous = g_file_get_relative_path (self->ostree_dir, current_deployment);
+      relative_previous = g_file_get_relative_path (self->osname_dir, current_deployment);
       g_assert (relative_previous);
-      if (symlink (relative_previous, ot_gfile_get_path_cached (tmp_previous_path)) < 0)
+      if (symlink (relative_previous, gs_file_get_path_cached (tmp_previous_path)) < 0)
         {
           ot_util_set_error_from_errno (error, errno);
           goto out;
         }
     }
 
-  tmp_current_path = g_file_get_child (self->ostree_dir, "tmp-current");
-  (void) ot_gfile_unlink (tmp_current_path, NULL, NULL);
+  tmp_current_path = g_file_get_child (self->osname_dir, "tmp-current");
+  (void) gs_file_unlink (tmp_current_path, NULL, NULL);
 
-  if (symlink (relative_current, ot_gfile_get_path_cached (tmp_current_path)) < 0)
+  if (symlink (relative_current, gs_file_get_path_cached (tmp_current_path)) < 0)
     {
       ot_util_set_error_from_errno (error, errno);
       goto out;
     }
 
-  tmp_current_etc_path = g_file_get_child (self->ostree_dir, "tmp-current-etc");
-  (void) ot_gfile_unlink (tmp_current_etc_path, NULL, NULL);
-  if (symlink (relative_current_etc, ot_gfile_get_path_cached (tmp_current_etc_path)) < 0)
-    {
-      ot_util_set_error_from_errno (error, errno);
-      goto out;
-    }
-
-  if (!ot_gfile_rename (tmp_current_path, current_path,
-                        cancellable, error))
-    goto out;
-  if (!ot_gfile_rename (tmp_current_etc_path, current_etc_path,
-                        cancellable, error))
+  if (!gs_file_rename (tmp_current_path, current_path,
+                       cancellable, error))
     goto out;
 
   if (tmp_previous_path)
     {
-      if (!ot_gfile_rename (tmp_previous_path, previous_path,
-                            cancellable, error))
+      if (!gs_file_rename (tmp_previous_path, previous_path,
+                           cancellable, error))
         goto out;
     }
 
-  g_print ("ostadmin: %s set to %s\n", ot_gfile_get_path_cached (current_path),
+  g_print ("ostadmin: %s set to %s\n", gs_file_get_path_cached (current_path),
            relative_current);
 
   ret = TRUE;
@@ -173,7 +163,7 @@ on_checkout_complete (GObject         *object,
 /**
  * ensure_unlinked:
  *
- * Like ot_gfile_unlink(), but return successfully if the file doesn't
+ * Like gs_file_unlink(), but return successfully if the file doesn't
  * exist.
  */
 static gboolean
@@ -184,7 +174,7 @@ ensure_unlinked (GFile         *path,
   gboolean ret = FALSE;
   GError *temp_error = NULL;
 
-  if (!ot_gfile_unlink (path, cancellable, &temp_error))
+  if (!gs_file_unlink (path, cancellable, &temp_error))
     {
       if (g_error_matches (temp_error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
         {
@@ -218,6 +208,7 @@ copy_one_config_file (OtAdminDeploy      *self,
                       GError            **error)
 {
   gboolean ret = FALSE;
+  ot_lobj GFileInfo *src_info = NULL;
   ot_lobj GFile *dest = NULL;
   ot_lobj GFile *parent = NULL;
   ot_lfree char *relative_path = NULL;
@@ -226,15 +217,52 @@ copy_one_config_file (OtAdminDeploy      *self,
   g_assert (relative_path);
   dest = g_file_resolve_relative_path (new_etc, relative_path);
 
-  parent = g_file_get_parent (dest);
-
-  /* FIXME actually we need to copy permissions and xattrs */
-  if (!ot_gfile_ensure_directory (parent, TRUE, error))
+  src_info = g_file_query_info (src, OSTREE_GIO_FAST_QUERYINFO, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                cancellable, error);
+  if (!src_info)
     goto out;
 
-  if (!g_file_copy (src, dest, G_FILE_COPY_OVERWRITE | G_FILE_COPY_NOFOLLOW_SYMLINKS | G_FILE_COPY_ALL_METADATA,
-                    cancellable, NULL, NULL, error))
-    goto out;
+  if (g_file_info_get_file_type (src_info) == G_FILE_TYPE_DIRECTORY)
+    {
+      ot_lobj GFileEnumerator *src_enum = NULL;
+      ot_lobj GFileInfo *child_info = NULL;
+      GError *temp_error = NULL;
+
+      /* FIXME actually we need to copy permissions and xattrs */
+      if (!gs_file_ensure_directory (dest, TRUE, cancellable, error))
+        goto out;
+
+      src_enum = g_file_enumerate_children (src, OSTREE_GIO_FAST_QUERYINFO,
+                                            G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                            cancellable, error);
+
+      while ((child_info = g_file_enumerator_next_file (src_enum, cancellable, error)) != NULL)
+        {
+          ot_lobj GFile *child = g_file_get_child (src, g_file_info_get_name (child_info));
+
+          if (!copy_one_config_file (self, orig_etc, modified_etc, new_etc, child,
+                                     cancellable, error))
+            goto out;
+        }
+      g_clear_object (&child_info);
+      if (temp_error != NULL)
+        {
+          g_propagate_error (error, temp_error);
+          goto out;
+        }
+    }
+  else
+    {
+      parent = g_file_get_parent (dest);
+
+      /* FIXME actually we need to copy permissions and xattrs */
+      if (!gs_file_ensure_directory (parent, TRUE, cancellable, error))
+        goto out;
+      
+      if (!g_file_copy (src, dest, G_FILE_COPY_OVERWRITE | G_FILE_COPY_NOFOLLOW_SYMLINKS | G_FILE_COPY_ALL_METADATA,
+                        cancellable, NULL, NULL, error))
+        goto out;
+    }
 
   ret = TRUE;
  out:
@@ -327,7 +355,7 @@ merge_etc_changes (OtAdminDeploy  *self,
  * deploy_tree:
  *
  * Look up @revision in the repository, and check it out in
- * OSTREE_DIR/deploy/DEPLOY_TARGET.
+ * OSTREE_DIR/deploy/OS/DEPLOY_TARGET.
  *
  * Merge configuration changes from the old deployment, if any.
  *
@@ -343,7 +371,8 @@ deploy_tree (OtAdminDeploy     *self,
              GError           **error)
 {
   gboolean ret = FALSE;
-  ot_lobj GFile *deploy_dir = NULL;
+  gs_free char *current_deployment_ref = NULL;
+  gs_free char *previous_deployment_ref = NULL;
   ot_lfree char *deploy_target_fullname = NULL;
   ot_lfree char *deploy_target_fullname_tmp = NULL;
   ot_lobj GFile *deploy_target_path = NULL;
@@ -353,6 +382,7 @@ deploy_tree (OtAdminDeploy     *self,
   ot_lobj GFile *deploy_target_default_etc_path = NULL;
   ot_lobj GFile *deploy_parent = NULL;
   ot_lobj GFile *previous_deployment = NULL;
+  ot_lfree char *previous_deployment_revision = NULL;
   ot_lobj GFile *previous_deployment_etc = NULL;
   ot_lobj GFile *previous_deployment_etc_default = NULL;
   ot_lobj OstreeRepoFile *root = NULL;
@@ -367,7 +397,16 @@ deploy_tree (OtAdminDeploy     *self,
   if (!revision)
     revision = deploy_target;
 
-  deploy_dir = g_file_get_child (self->ostree_dir, "deploy");
+  current_deployment_ref = g_strdup_printf ("deployment/%s/current", self->osname);
+  previous_deployment_ref = g_strdup_printf ("deployment/%s/previous", self->osname);
+
+  if (!g_file_query_exists (self->osname_dir, cancellable))
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "No OS \"%s\" found in \"%s\"", self->osname,
+                   gs_file_get_path_cached (self->osname_dir));
+      goto out;
+    }
 
   if (!ostree_repo_resolve_rev (self->repo, revision, FALSE, &resolved_commit, error))
     goto out;
@@ -383,20 +422,20 @@ deploy_tree (OtAdminDeploy     *self,
     goto out;
 
   deploy_target_fullname = g_strconcat (deploy_target, "-", resolved_commit, NULL);
-  deploy_target_path = g_file_resolve_relative_path (deploy_dir, deploy_target_fullname);
+  deploy_target_path = g_file_resolve_relative_path (self->osname_dir, deploy_target_fullname);
 
   deploy_target_fullname_tmp = g_strconcat (deploy_target_fullname, ".tmp", NULL);
-  deploy_target_path_tmp = g_file_resolve_relative_path (deploy_dir, deploy_target_fullname_tmp);
+  deploy_target_path_tmp = g_file_resolve_relative_path (self->osname_dir, deploy_target_fullname_tmp);
 
   deploy_parent = g_file_get_parent (deploy_target_path);
-  if (!ot_gfile_ensure_directory (deploy_parent, TRUE, error))
+  if (!gs_file_ensure_directory (deploy_parent, TRUE, cancellable, error))
     goto out;
 
   deploy_target_etc_name = g_strconcat (deploy_target, "-", resolved_commit, "-etc", NULL);
-  deploy_target_etc_path = g_file_resolve_relative_path (deploy_dir, deploy_target_etc_name);
+  deploy_target_etc_path = g_file_resolve_relative_path (self->osname_dir, deploy_target_etc_name);
 
   /* Delete any previous temporary data */
-  if (!ot_gio_shutil_rm_rf (deploy_target_path_tmp, cancellable, error))
+  if (!gs_shutil_rm_rf (deploy_target_path_tmp, cancellable, error))
     goto out;
 
   existing_checkout_info = g_file_query_info (deploy_target_path, OSTREE_GIO_FAST_QUERYINFO,
@@ -406,9 +445,9 @@ deploy_tree (OtAdminDeploy     *self,
     {
       if (opt_force)
         {
-          if (!ot_gio_shutil_rm_rf (deploy_target_path, cancellable, error))
+          if (!gs_shutil_rm_rf (deploy_target_path, cancellable, error))
             goto out;
-          if (!ot_gio_shutil_rm_rf (deploy_target_etc_path, cancellable, error))
+          if (!gs_shutil_rm_rf (deploy_target_etc_path, cancellable, error))
             goto out;
           
           skip_checkout = FALSE;
@@ -427,7 +466,7 @@ deploy_tree (OtAdminDeploy     *self,
       goto out;
     }
 
-  if (!ot_admin_get_current_deployment (self->ostree_dir, &previous_deployment,
+  if (!ot_admin_get_current_deployment (self->ostree_dir, self->osname, &previous_deployment,
                                         cancellable, error))
     goto out;
   if (previous_deployment)
@@ -435,7 +474,7 @@ deploy_tree (OtAdminDeploy     *self,
       ot_lfree char *etc_name;
       ot_lobj GFile *parent;
 
-      etc_name = g_strconcat (ot_gfile_get_basename_cached (previous_deployment), "-etc", NULL);
+      etc_name = g_strconcat (gs_file_get_basename_cached (previous_deployment), "-etc", NULL);
       parent = g_file_get_parent (previous_deployment);
 
       previous_deployment_etc = g_file_get_child (parent, etc_name);
@@ -445,6 +484,10 @@ deploy_tree (OtAdminDeploy     *self,
         g_clear_object (&previous_deployment_etc);
       else
         previous_deployment_etc_default = g_file_get_child (previous_deployment, "etc");
+
+      if (!ostree_repo_resolve_rev (self->repo, current_deployment_ref, TRUE,
+                                    &previous_deployment_revision, error))
+        goto out;
     }
 
 
@@ -453,7 +496,7 @@ deploy_tree (OtAdminDeploy     *self,
       ProcessOneCheckoutData checkout_data;
 
       g_print ("ostadmin: Creating deployment %s\n",
-               ot_gfile_get_path_cached (deploy_target_path));
+               gs_file_get_path_cached (deploy_target_path));
 
       memset (&checkout_data, 0, sizeof (checkout_data));
       checkout_data.loop = g_main_loop_new (NULL, TRUE);
@@ -475,14 +518,14 @@ deploy_tree (OtAdminDeploy     *self,
 
       deploy_target_default_etc_path = ot_gfile_get_child_strconcat (deploy_target_path_tmp, "etc", NULL);
 
-      if (!ot_gio_shutil_rm_rf (deploy_target_etc_path, cancellable, error))
+      if (!gs_shutil_rm_rf (deploy_target_etc_path, cancellable, error))
         goto out;
 
-      if (!ot_gio_shutil_cp_a (deploy_target_default_etc_path, deploy_target_etc_path,
-                               cancellable, error))
+      if (!gs_shutil_cp_a (deploy_target_default_etc_path, deploy_target_etc_path,
+                           cancellable, error))
         goto out;
 
-      g_print ("ostadmin: Created %s\n", ot_gfile_get_path_cached (deploy_target_etc_path));
+      g_print ("ostadmin: Created %s\n", gs_file_get_path_cached (deploy_target_etc_path));
 
       if (previous_deployment_etc)
         {
@@ -494,8 +537,21 @@ deploy_tree (OtAdminDeploy     *self,
       else
         g_print ("ostadmin: No previous deployment; therefore, no configuration changes to merge\n");
 
-      if (!ot_gfile_rename (deploy_target_path_tmp, deploy_target_path,
-                            cancellable, error))
+      if (!gs_file_rename (deploy_target_path_tmp, deploy_target_path,
+                           cancellable, error))
+        goto out;
+    }
+
+  /* Write out a ref so that any "ostree prune" on the raw repo
+   * doesn't GC the currently deployed tree.
+   */
+  if (!ostree_repo_write_ref (self->repo, NULL, current_deployment_ref,
+                              resolved_commit, error))
+    goto out;
+  if (previous_deployment_revision != NULL)
+    {
+      if (!ostree_repo_write_ref (self->repo, NULL, previous_deployment_ref,
+                                  previous_deployment_revision, error))
         goto out;
     }
 
@@ -521,18 +577,28 @@ do_update_kernel (OtAdminDeploy     *self,
                   GError           **error)
 {
   gboolean ret = FALSE;
-  ot_lptrarray GPtrArray *args = NULL;
+  gs_unref_object GSSubprocess *proc = NULL;
+  gs_unref_ptrarray GPtrArray *args = NULL;
 
   args = g_ptr_array_new ();
   ot_ptrarray_add_many (args, "ostree", "admin",
-                        "--ostree-dir", ot_gfile_get_path_cached (self->ostree_dir),
+                        "--ostree-dir", gs_file_get_path_cached (self->ostree_dir),
                         "update-kernel",
-                        ot_gfile_get_path_cached (deploy_path), NULL);
+                        self->osname,
+                        gs_file_get_path_cached (deploy_path), NULL);
+  if (opt_no_kernel)
+    g_ptr_array_add (args, "--modules-only");
+  if (opt_host_kernel)
+    g_ptr_array_add (args, "--host-kernel");
   g_ptr_array_add (args, NULL);
 
-  if (!ot_spawn_sync_checked (ot_gfile_get_path_cached (self->ostree_dir),
-                              (char**)args->pdata, NULL, G_SPAWN_SEARCH_PATH,
-                              NULL, NULL, NULL, NULL, error))
+  proc = gs_subprocess_new_simple_argv ((char**)args->pdata,
+                                        GS_SUBPROCESS_STREAM_DISPOSITION_INHERIT,
+                                        GS_SUBPROCESS_STREAM_DISPOSITION_INHERIT,
+                                        cancellable, error);
+  if (!proc)
+    goto out;
+  if (!gs_subprocess_wait_sync_check (proc, cancellable, error))
     goto out;
 
   ret = TRUE;
@@ -540,7 +606,6 @@ do_update_kernel (OtAdminDeploy     *self,
   return ret;
 }
 
-
 gboolean
 ot_admin_builtin_deploy (int argc, char **argv, GFile *ostree_dir, GError **error)
 {
@@ -550,22 +615,23 @@ ot_admin_builtin_deploy (int argc, char **argv, GFile *ostree_dir, GError **erro
   gboolean ret = FALSE;
   ot_lobj GFile *repo_path = NULL;
   ot_lobj GFile *deploy_path = NULL;
+  const char *osname = NULL;
   const char *deploy_target = NULL;
   const char *revision = NULL;
   __attribute__((unused)) GCancellable *cancellable = NULL;
 
   memset (self, 0, sizeof (*self));
 
-  context = g_option_context_new ("NAME [REVISION] - Check out revision NAME (or REVISION as NAME)");
+  context = g_option_context_new ("OSNAME TREENAME [REVISION] - In operating system OS, check out revision TREENAME (or REVISION as TREENAME)");
 
   g_option_context_add_main_entries (context, options, NULL);
 
   if (!g_option_context_parse (context, &argc, &argv, error))
     goto out;
 
-  if (argc < 2)
+  if (argc < 3)
     {
-      ot_util_usage_error (context, "NAME must be specified", error);
+      ot_util_usage_error (context, "OSNAME and TREENAME must be specified", error);
       goto out;
     }
 
@@ -579,24 +645,26 @@ ot_admin_builtin_deploy (int argc, char **argv, GFile *ostree_dir, GError **erro
   if (!ostree_repo_check (self->repo, error))
     goto out;
 
-  deploy_target = argv[1];
-  if (argc > 2)
-    revision = argv[2];
+  osname = argv[1];
+  deploy_target = argv[2];
+  if (argc > 3)
+    revision = argv[3];
 
+  self->osname = g_strdup (osname);
+  self->osname_dir = ot_gfile_get_child_build_path (self->ostree_dir, "deploy", osname, NULL);
   if (!deploy_tree (self, deploy_target, revision, &deploy_path,
                     cancellable, error))
     goto out;
 
-  if (!opt_no_kernel)
-    {
-      if (!do_update_kernel (self, deploy_path, cancellable, error))
-        goto out;
-    }
+  if (!do_update_kernel (self, deploy_path, cancellable, error))
+    goto out;
 
   ret = TRUE;
  out:
   g_clear_object (&self->repo);
+  g_free (self->osname);
   g_clear_object (&self->ostree_dir);
+  g_clear_object (&self->osname_dir);
   if (context)
     g_option_context_free (context);
   return ret;
