@@ -28,52 +28,33 @@
 #include <glib/gi18n.h>
 #include <glib/gprintf.h>
 
-static gboolean verbose;
-static gboolean delete;
-static int depth = -1;
+static gboolean opt_no_prune;
+static gint opt_depth = -1;
+static gboolean opt_refs_only;
 
 static GOptionEntry options[] = {
-  { "verbose", 0, 0, G_OPTION_ARG_NONE, &verbose, "Display progress", NULL },
-  { "depth", 0, 0, G_OPTION_ARG_INT, &depth, "Only traverse commit objects by this count", NULL },
-  { "delete", 0, 0, G_OPTION_ARG_NONE, &delete, "Remove no longer reachable objects", NULL },
+  { "no-prune", 0, 0, G_OPTION_ARG_NONE, &opt_no_prune, "Only display unreachable objects; don't delete", NULL },
+  { "refs-only", 0, 0, G_OPTION_ARG_NONE, &opt_refs_only, "Only compute reachability via refs", NULL },
+  { "depth", 0, 0, G_OPTION_ARG_INT, &opt_depth, "Only traverse DEPTH parents for each commit (default: -1=infinite)", "DEPTH" },
   { NULL }
 };
 
-static void
-log_verbose (const char  *fmt,
-             ...) G_GNUC_PRINTF (1, 2);
-
-static void
-log_verbose (const char  *fmt,
-             ...)
-{
-  va_list args;
-
-  if (!verbose)
-    return;
-
-  va_start (args, fmt);
-  
-  g_vprintf (fmt, args);
-  g_print ("\n");
-
-  va_end (args);
-}
-
 typedef struct {
   OstreeRepo *repo;
   GHashTable *reachable;
-  guint n_reachable;
-  guint n_unreachable;
+  guint n_reachable_meta;
+  guint n_reachable_content;
+  guint n_unreachable_meta;
+  guint n_unreachable_content;
+  guint64 freed_bytes;
 } OtPruneData;
 
-
 static gboolean
-prune_loose_object (OtPruneData    *data,
-                    const char    *checksum,
-                    OstreeObjectType objtype,
-                    GCancellable    *cancellable,
-                    GError         **error)
+maybe_prune_loose_object (OtPruneData    *data,
+                          const char    *checksum,
+                          OstreeObjectType objtype,
+                          GCancellable    *cancellable,
+                          GError         **error)
 {
   gboolean ret = FALSE;
   ot_lvariant GVariant *key = NULL;
@@ -85,20 +66,32 @@ prune_loose_object (OtPruneData    *data,
 
   if (!g_hash_table_lookup_extended (data->reachable, key, NULL, NULL))
     {
-      if (delete)
+      if (!opt_no_prune)
         {
-          if (!ot_gfile_unlink (objf, cancellable, error))
+          ot_lobj GFileInfo *info = NULL;
+
+          if ((info = g_file_query_info (objf, OSTREE_GIO_FAST_QUERYINFO,
+                                         G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                         cancellable, error)) == NULL)
             goto out;
-          g_print ("Deleted: %s.%s\n", checksum, ostree_object_type_to_string (objtype));
+
+          if (!gs_file_unlink (objf, cancellable, error))
+            goto out;
+
+          data->freed_bytes += g_file_info_get_size (info);
         }
+      if (OSTREE_OBJECT_TYPE_IS_META (objtype))
+        data->n_unreachable_meta++;
       else
-        {
-          g_print ("Unreachable: %s.%s\n", checksum, ostree_object_type_to_string (objtype));
-        }
-      data->n_unreachable++;
+        data->n_unreachable_content++;
     }
   else
-    data->n_reachable++;
+    {
+      if (OSTREE_OBJECT_TYPE_IS_META (objtype))
+        data->n_reachable_meta++;
+      else
+        data->n_reachable_content++;
+    }
 
   ret = TRUE;
  out:
@@ -132,29 +125,47 @@ ostree_builtin_prune (int argc, char **argv, GFile *repo_path, GError **error)
 
   data.repo = repo;
   data.reachable = ostree_traverse_new_reachable ();
-  data.n_reachable = 0;
-  data.n_unreachable = 0;
 
-  if (!ostree_repo_list_all_refs (repo, &all_refs, cancellable, error))
-    goto out;
-
-  g_hash_table_iter_init (&hash_iter, all_refs);
-
-  while (g_hash_table_iter_next (&hash_iter, &key, &value))
+  if (opt_refs_only)
     {
-      const char *name = key;
-      const char *checksum = value;
-
-      log_verbose ("Computing reachable, currently %u total, from %s: %s", g_hash_table_size (data.reachable), name, checksum);
-      if (!ostree_traverse_commit (repo, checksum, depth, data.reachable, cancellable, error))
+      if (!ostree_repo_list_all_refs (repo, &all_refs, cancellable, error))
         goto out;
+      
+      g_hash_table_iter_init (&hash_iter, all_refs);
+      
+      while (g_hash_table_iter_next (&hash_iter, &key, &value))
+        {
+          const char *checksum = value;
+          
+          // g_print ("Computing reachable, currently %u total, from %s: %s\n", g_hash_table_size (data.reachable), name, checksum);
+          if (!ostree_traverse_commit (repo, checksum, opt_depth, data.reachable, cancellable, error))
+            goto out;
+        }
     }
 
   if (!ostree_repo_list_objects (repo, OSTREE_REPO_LIST_OBJECTS_ALL, &objects, cancellable, error))
     goto out;
 
-  g_hash_table_iter_init (&hash_iter, objects);
+  if (!opt_refs_only)
+    {
+      g_hash_table_iter_init (&hash_iter, objects);
+      while (g_hash_table_iter_next (&hash_iter, &key, &value))
+        {
+          GVariant *serialized_key = key;
+          const char *checksum;
+          OstreeObjectType objtype;
 
+          ostree_object_name_deserialize (serialized_key, &checksum, &objtype);
+
+          if (objtype != OSTREE_OBJECT_TYPE_COMMIT)
+            continue;
+          
+          if (!ostree_traverse_commit (repo, checksum, opt_depth, data.reachable, cancellable, error))
+            goto out;
+        }
+    }
+
+  g_hash_table_iter_init (&hash_iter, objects);
   while (g_hash_table_iter_next (&hash_iter, &key, &value))
     {
       GVariant *serialized_key = key;
@@ -162,20 +173,26 @@ ostree_builtin_prune (int argc, char **argv, GFile *repo_path, GError **error)
       const char *checksum;
       OstreeObjectType objtype;
       gboolean is_loose;
-
+      
       ostree_object_name_deserialize (serialized_key, &checksum, &objtype);
-
       g_variant_get_child (objdata, 0, "b", &is_loose);
 
-      if (is_loose)
-        {
-          if (!prune_loose_object (&data, checksum, objtype, cancellable, error))
-            goto out;
-        }
+      if (!is_loose)
+        continue;
+
+      if (!maybe_prune_loose_object (&data, checksum, objtype, cancellable, error))
+        goto out;
     }
 
-  g_print ("Total reachable: %u\n", data.n_reachable);
-  g_print ("Total unreachable: %u\n", data.n_unreachable);
+  g_print ("Total reachable: %u meta, %u content\n",
+           data.n_reachable_meta, data.n_reachable_content);
+  if (opt_no_prune)
+    g_print ("Total unreachable: %u meta, %u content\n",
+             data.n_unreachable_meta, data.n_unreachable_content);
+  else
+    g_print ("Freed %" G_GUINT64_FORMAT " bytes from %u meta, %u content objects\n",
+             data.freed_bytes, data.n_unreachable_meta, data.n_unreachable_content);
+    
 
   ret = TRUE;
  out:
