@@ -31,7 +31,17 @@ typedef struct {
   const char *name;
   gboolean (*fn) (int argc, char **argv, GFile *repo_path, GError **error);
   int flags; /* OstreeBuiltinFlags */
-} OstreeBuiltin;
+} OstreeCommand;
 
-int ostree_main (int    argc, char **argv, OstreeBuiltin  *builtins);
+void
+ostree_prep_builtin_argv (const char  *builtin,
+                          int          argc,
+                          char       **argv,
+                          int         *out_argc,
+                          char      ***out_argv);
 
+int ostree_main (int    argc, char **argv, OstreeCommand  *commands);
+
+int ostree_run (int argc, char **argv, OstreeCommand *commands, GError **error);
+
+int ostree_usage (char **argv, OstreeCommand *commands, gboolean is_error);
